"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * An event that fires its listeners in a synchronous fashion.
 */
var Event = /** @class */ (function () {
    /**
     * Create a new event.
     *
     * @param parent
     *   the parent that will be passed to listener as their `this`
     */
    function Event(parent) {
        var _this = this;
        this.parent = parent;
        var subscribable = function (listener) { return _this.subscribe(listener); };
        subscribable.subscribe = subscribable;
        subscribable.unsubscribe = function (listener) { return _this.unsubscribe(listener); };
        this.subscribable = subscribable;
    }
    /**
     * Emit this event. This will invoke all of the listeners with the passed
     * arguments.
     *
     * @param args
     */
    Event.prototype.emit = function () {
        var e_1, _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (Array.isArray(this.listeners)) {
            try {
                /*
                 * Array is present, iterate over array and invoke all of the
                 * listeners.
                 */
                for (var _b = __values(this.listeners), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var listener = _c.value;
                    listener.apply(this.parent, args);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        else if (this.listeners) {
            /*
             * Single listener is present, simply invoke the listener.
             */
            this.listeners.apply(this.parent, args);
        }
    };
    /**
     * Subscribe to this event using the given listener. The listener will
     * be invoked any time the event is emittred. The returned handle can be
     * used to unsubscribe.
     *
     * @param listener
     */
    Event.prototype.subscribe = function (listener) {
        if (Array.isArray(this.listeners)) {
            // Listeners is already an array, push onto the array
            this.listeners.push(listener);
        }
        else if (this.listeners) {
            this.listeners = [this.listeners, listener];
        }
        else {
            this.listeners = listener;
        }
        var self = this;
        return {
            unsubscribe: function () {
                return self.unsubscribe(listener);
            }
        };
    };
    /**
     * Unsubscribe a listener from this handler. The specified listener will
     * no longer be invoked when the event is emitted.
     *
     * @param listener
     */
    Event.prototype.unsubscribe = function (listener) {
        if (Array.isArray(this.listeners)) {
            /*
             * Array has been allocated, find the index of the listener and
             * then remove it from the array.
             */
            var idx = this.listeners.indexOf(listener);
            if (idx < 0)
                return false;
            this.listeners.splice(idx);
            /*
             * If the array is empty, remove it. Otherwise at this point the
             * array has already been allocated so keep the array in case a
             * subscription happens again.
             */
            if (this.listeners.length === 0) {
                this.listeners = undefined;
            }
            return true;
        }
        else if (this.listeners === listener) {
            /*
             * Single listener is present and its the current match. Reset
             * listeners property.
             */
            this.listeners = undefined;
            return true;
        }
        // Listener is not active
        return false;
    };
    return Event;
}());
exports.Event = Event;
//# sourceMappingURL=sync.js.map